# -*- coding: utf-8 -*-
from .autosig import autosig, Signature, signature, param
"""Top-level package for autosig."""

__all__ = ["autosig", "Signature", "signature", "param"]
__author__ = """Antonio Piccolboni"""
__email__ = 'autosig@piccolboni.info'
__version__ = '0.2.2'
